@@ -1,6 +1,6 @@
 /* Glom
  *
- * Copyright (C) 2001-2004 Murray Cumming
+ * Copyright (C) 2001-2009 Murray Cumming
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,18 +18,18 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "box_data_details.h"
-#include "../frame_glom.h" //For show_ok_dialog().
-#include <glom/libglom/data_structure/field.h>
-#include <glom/libglom/data_structure/relationship.h>
-#include <glom/libglom/data_structure/glomconversions.h>
-#include "dialog_layout_details.h"
-#include <glom/libglom/glade_utils.h>
-#include <glom/libglom/utils.h>
+#include "config.h"
+#include <glom/mode_data/box_data_details.h>
+#include <glom/frame_glom.h> //For show_ok_dialog().
+#include <libglom/data_structure/field.h>
+#include <libglom/data_structure/relationship.h>
+#include <libglom/data_structure/glomconversions.h>
+#include <glom/mode_design/layout/dialog_layout_details.h>
+#include <glom/glade_utils.h>
+#include <glom/utils_ui.h>
 #include <glom/glom_privs.h>
-#include "../xsl_utils.h"
-#include <bakery/App/App_Gtk.h> //For util_bold_message().
-#include "../python_embed/glom_python.h"
+#include <glom/xsl_utils.h>
+#include <glom/python_embed/glom_python.h>
 #include <sstream> //For stringstream
 #include <glibmm/i18n.h>
 
@@ -37,83 +37,67 @@ namespace Glom
 {
 
 Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
-: m_HBox(false, Utils::DEFAULT_SPACING_SMALL),
-  m_HBox_Sidebar (false, Utils::DEFAULT_SPACING_SMALL),
+: m_hbox_content(false, Utils::DEFAULT_SPACING_SMALL),
+#ifndef GLOM_ENABLE_MAEMO
+  m_hbox_buttons(false, Utils::DEFAULT_SPACING_SMALL),
   m_Button_New(Gtk::Stock::ADD),
   m_Button_Del(Gtk::Stock::DELETE),
   m_Button_Nav_First(Gtk::Stock::GOTO_FIRST),
   m_Button_Nav_Prev(Gtk::Stock::GO_BACK),
   m_Button_Nav_Next(Gtk::Stock::GO_FORWARD),
   m_Button_Nav_Last(Gtk::Stock::GOTO_LAST),
+#endif //GLOM_ENABLE_MAEMO
   m_bDoNotRefreshRelated(false),
   m_ignore_signals(true)
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-  ,m_design_mode(false)
+  , m_design_mode(false)
 #endif
 {
   m_layout_name = "details";
 
-  m_Button_New.set_tooltip_text(_("Create a new record."));
-  m_Button_Del.set_tooltip_text(_("Remove this record."));
-  m_Button_Nav_First.set_tooltip_text(_("View the first record in the list."));
-  m_Button_Nav_Prev.set_tooltip_text(_("View the previous record in the list."));
-  m_Button_Nav_Next.set_tooltip_text(_("View the next record in the list."));
-  m_Button_Nav_Last.set_tooltip_text(_("View the last record in the list."));
-  
-
   add_view(&m_FlowTable); //Allow this to access the document too.
 
   m_FlowTable.set_columns_count(1); //Sub-groups will have multiple columns (by default, there is one sub-group, with 2 columns).
-  m_FlowTable.set_padding(Utils::DEFAULT_SPACING_SMALL);
+
+  #ifndef GLOM_ENABLE_MAEMO
+  m_FlowTable.set_column_padding(Utils::DEFAULT_SPACING_SMALL); //The default anyway.
+  m_FlowTable.set_row_padding(Utils::DEFAULT_SPACING_SMALL); //The default anyway.
+  #else
+  m_FlowTable.set_row_padding(0); //The hildon buttons and entries have their own default padding.
+  m_FlowTable.set_column_padding(HILDON_MARGIN_DOUBLE); //As per the UI specs.
+  #endif
 
   //m_strHint = _("When you change the data in a field the database is updated immediately.\n Click [New] to add a new record.\n Leave automatic ID fields empty - they will be filled for you.");
 
 
   //m_ScrolledWindow.set_border_width(Utils::DEFAULT_SPACING_SMALL);
 #ifdef GLOM_ENABLE_MAEMO
-  m_ScrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC); /* Allow horizontal scrolling in maemo because the screen is rather small and there might be some database UIs that don't fit horizontally. Such a UI may be concidered non-maemo-friendly, but it can still be fully viewed this way. */
+  // Allow horizontal scrolling in maemo because the screen is rather small and
+  // there might be some database UIs that don't fit horizontally. Such a UI may
+  // be considered non-maemo-friendly, but it can still be fully viewed this way.
+  // TODO: Prevent the need for horizontal scrolling.
+  m_ScrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
 #else
-  m_ScrolledWindow.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC); /* Allow vertical scrolling, but never scroll horizontally. */
+  // Allow vertical scrolling, but never scroll horizontally:
+  m_ScrolledWindow.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);
 #endif
   m_ScrolledWindow.set_shadow_type(Gtk::SHADOW_NONE); //SHADOW_IN is Recommended by the GNOME HIG, but looks odd.
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-  m_HBox_Sidebar.pack_start(m_Dragbar, Gtk::PACK_SHRINK);
+  m_hbox_content.pack_start(m_Dragbar, Gtk::PACK_SHRINK);
   m_Dragbar.hide();
 #endif
 
-  m_HBox_Sidebar.pack_start(m_ScrolledWindow);
+  m_hbox_content.pack_start(m_ScrolledWindow);
   m_ScrolledWindow.add(m_FlowTable);
   // The FlowTable does not support native scrolling, so gtkmm adds it to a
   // viewport first that also has some shadow we do not want.
   Gtk::Viewport* viewport = dynamic_cast<Gtk::Viewport*>(m_FlowTable.get_parent());
-  if(viewport) viewport->set_shadow_type(Gtk::SHADOW_NONE);
+  if(viewport)
+    viewport->set_shadow_type(Gtk::SHADOW_NONE);
 
-  //Add or delete record:
-  m_HBox.pack_start(m_Button_New, Gtk::PACK_SHRINK);
-  m_HBox.pack_start(m_Button_Del,  Gtk::PACK_SHRINK);
 
-   //Link buttons to handlers:
-  m_Button_New.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_new));
-  m_Button_Del.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_del));
-
-  //Navigation:
-  if(bWithNavButtons)
-  {
-    m_HBox.pack_end(m_Button_Nav_Last, Gtk::PACK_SHRINK);
-    m_HBox.pack_end(m_Button_Nav_Next, Gtk::PACK_SHRINK);
-    m_HBox.pack_end(m_Button_Nav_Prev, Gtk::PACK_SHRINK);
-    m_HBox.pack_end(m_Button_Nav_First, Gtk::PACK_SHRINK);
-  }
-
-  //Link buttons to handlers:
-  m_Button_Nav_First.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_first));
-  m_Button_Nav_Prev.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_prev));
-  m_Button_Nav_Next.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_next));
-  m_Button_Nav_Last.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_last));
-
-  pack_start(m_HBox_Sidebar);
-  pack_start(m_HBox, Gtk::PACK_SHRINK);
+  pack_start(m_hbox_content);
 
   m_FlowTable.signal_field_edited().connect( sigc::mem_fun(*this,  &Box_Data_Details::on_flowtable_field_edited) );
   m_FlowTable.signal_field_open_details_requested().connect( sigc::mem_fun(*this,  &Box_Data_Details::on_flowtable_field_open_details_requested) );
@@ -130,6 +114,41 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
 
   m_FlowTable.signal_script_button_clicked().connect( sigc::mem_fun(*this, &Box_Data_Details::on_flowtable_script_button_clicked) );
 
+
+#ifndef GLOM_ENABLE_MAEMO
+  m_Button_New.set_tooltip_text(_("Create a new record."));
+  m_Button_Del.set_tooltip_text(_("Remove this record."));
+  m_Button_Nav_First.set_tooltip_text(_("View the first record in the list."));
+  m_Button_Nav_Prev.set_tooltip_text(_("View the previous record in the list."));
+  m_Button_Nav_Next.set_tooltip_text(_("View the next record in the list."));
+  m_Button_Nav_Last.set_tooltip_text(_("View the last record in the list."));
+
+  //Add or delete record:
+  m_hbox_buttons.pack_start(m_Button_New, Gtk::PACK_SHRINK);
+  m_hbox_buttons.pack_start(m_Button_Del,  Gtk::PACK_SHRINK);
+
+  //Link buttons to handlers:
+  m_Button_New.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_new));
+  m_Button_Del.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_del));
+
+  //Navigation:
+  if(bWithNavButtons)
+  {
+    m_hbox_buttons.pack_end(m_Button_Nav_Last, Gtk::PACK_SHRINK);
+    m_hbox_buttons.pack_end(m_Button_Nav_Next, Gtk::PACK_SHRINK);
+    m_hbox_buttons.pack_end(m_Button_Nav_Prev, Gtk::PACK_SHRINK);
+    m_hbox_buttons.pack_end(m_Button_Nav_First, Gtk::PACK_SHRINK);
+  }
+
+  //Link buttons to handlers:
+  m_Button_Nav_First.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_first));
+  m_Button_Nav_Prev.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_prev));
+  m_Button_Nav_Next.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_next));
+  m_Button_Nav_Last.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_last));
+
+  pack_start(m_hbox_buttons, Gtk::PACK_SHRINK);
+#endif //GLOM_ENABLE_MAEMO
+
   m_ignore_signals = false;
 }
 
@@ -157,7 +176,7 @@ void Box_Data_Details::set_found_set_from_primary_key_value()
   if(!m_primary_key_value.is_null())
   {
     //TODO: Use a SQL parameter instead of using sql().
-    m_found_set.m_where_clause = "\"" + m_table_name + "\".\"" + m_field_primary_key->get_name() + 
+    m_found_set.m_where_clause = "\"" + m_table_name + "\".\"" + m_field_primary_key->get_name() +
       "\" = " + m_field_primary_key->sql(m_primary_key_value);
     //std::cout << "  DEBUG: Box_Data_Details::set_primary_key_value(): m_found_set.m_where_clause = " << m_found_set.m_where_clause << std::endl;
   }
@@ -169,13 +188,13 @@ bool Box_Data_Details::init_db_details(const FoundSet& found_set, const Glib::us
 
   m_primary_key_value = primary_key_value;
   m_field_primary_key = get_field_primary_key_for_table(found_set.m_table_name);
-   
+
   const bool result = Box_Data::init_db_details(found_set, layout_platform); //Calls create_layout(), then fill_from_database()
 
   //This is not used much, but we create it anyway:
   m_found_set = found_set; //Not used much.
   set_found_set_from_primary_key_value();
- 
+
   return result;
 }
 
@@ -193,25 +212,27 @@ bool Box_Data_Details::refresh_data_from_database_blank()
 
 void Box_Data_Details::create_layout()
 {
-  Bakery::BusyCursor busy_cursor(get_app_window());
+  BusyCursor busy_cursor(get_app_window());
 
   Box_Data::create_layout(); //Fills m_TableFields.
 
   //Remove existing child widgets:
   m_FlowTable.remove_all();
 
-  Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
+  Document* document = dynamic_cast<Document*>(get_document());
   if(document)
   {
     m_FlowTable.set_table(m_table_name); //This allows portals to get full Relationship information
 
     //This map of layout groups will also contain the field information from the database:
-    Document_Glom::type_list_layout_groups layout_groups = get_data_layout_groups(m_layout_name, m_layout_platform);
+    Document::type_list_layout_groups layout_groups = get_data_layout_groups(m_layout_name, m_layout_platform);
 
-    for(Document_Glom::type_list_layout_groups::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
+    for(Document::type_list_layout_groups::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
     {
       m_FlowTable.add_layout_group(*iter);
     }
+
+    m_FlowTable.align_child_group_labels();
   }
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
@@ -230,7 +251,7 @@ bool Box_Data_Details::fill_from_database()
 
   bool bResult = false;
 
-  Bakery::BusyCursor busy_cursor(get_app_window());
+  BusyCursor busy_cursor(get_app_window());
 
   const bool primary_key_is_empty = Conversions::value_is_empty(m_primary_key_value);
   if(!primary_key_is_empty)
@@ -242,7 +263,7 @@ bool Box_Data_Details::fill_from_database()
     return false;
   }
 
-  //TODO: This should keep the connection open, so we don't need to 
+  //TODO: This should keep the connection open, so we don't need to
   //reconnect many times..
   sharedptr<SharedConnection> sharedconnection;
 
@@ -285,15 +306,17 @@ bool Box_Data_Details::fill_from_database()
       Privileges table_privs = Privs::get_current_privs(m_table_name);
 
       //Enable/Disable record creation and deletion:
+      #ifndef GLOM_ENABLE_MAEMO
       m_Button_New.set_sensitive(table_privs.m_create);
       m_Button_Del.set_sensitive(table_privs.m_delete);
+      #endif //GLOM_ENABLE_MAEMO
 
       if(table_privs.m_view)
       {
         //Add extra possibly-non-visible columns that we need:
         sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
         layout_item->set_full_field_details(m_field_primary_key);
-        
+
         //Get the primary key index, adding the primary key if necessary:
         //TODO_Performance: Do this for create_layout() only, instead of repeating it for each refresh?:
         int index_primary_key = -1; //Arbitrary default.
@@ -314,11 +337,11 @@ bool Box_Data_Details::fill_from_database()
             sharedptr<const LayoutItem_Field> element = fieldsToGet[i];
             if(!element)
               continue;
-            
+
             if(element->get_name() != layout_item->get_name())
               continue;
-            
-            //Compare the relationship and related relationshp:
+
+            //Compare the relationship and related relationship:
             sharedptr<const UsesRelationship> uses_a = layout_item;
             sharedptr<const UsesRelationship> uses_b = element;
             if(*uses_a == *uses_b)
@@ -337,11 +360,11 @@ bool Box_Data_Details::fill_from_database()
 
         if((result && result->get_n_rows()) || primary_key_is_empty) //either a working result or no result needed.
         {
-          const Document_Glom* pDoc = dynamic_cast<const Document_Glom*>(get_document());
+          const Document* pDoc = dynamic_cast<const Document*>(get_document());
           if(pDoc)
           {
             //Get glom-specific field info:
-            //Document_Glom::type_vecFields vecFields = pDoc->get_table_fields(m_table_name);
+            //Document::type_vec_fields vecFields = pDoc->get_table_fields(m_table_name);
 
             const int row_number = 0; //The only row.
             int cols_count = 0;
@@ -355,7 +378,14 @@ bool Box_Data_Details::fill_from_database()
             {
               if(index_primary_key < cols_count)
               {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
                 m_primary_key_value = result->get_value_at(index_primary_key, row_number);
+#else
+              {
+                std::auto_ptr<Glib::Error> value_error;
+                m_primary_key_value = result->get_value_at(index_primary_key, row_number, value_error);
+              }
+#endif
                 set_found_set_from_primary_key_value();
               }
             }
@@ -369,7 +399,14 @@ bool Box_Data_Details::fill_from_database()
               Gnome::Gda::Value value;
 
               if(!primary_key_is_empty)
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
                 value = result->get_value_at(i, row_number);
+#else
+              {
+                std::auto_ptr<Glib::Error> value_error;
+                value = result->get_value_at(i, row_number, value_error);
+              }
+#endif
               else
               {
                 value = Conversions::get_empty_value(layout_item->get_glom_type());
@@ -404,7 +441,7 @@ void Box_Data_Details::on_button_new()
   {
     //Warn the user that they won't see anything if there are no fields on the layout,
     //doing an extra check:
-    Document_Glom* document = get_document();
+    Document* document = get_document();
     if( document && !(document->get_data_layout_groups_have_any_fields(m_layout_name, m_table_name, m_layout_platform)) )
     {
       Gtk::Window* parent_window = get_app_window();
@@ -431,12 +468,13 @@ void Box_Data_Details::on_button_new()
   }
 }
 
+#ifndef GLOM_ENABLE_MAEMO
 void Box_Data_Details::on_button_del()
 {
   if( Conversions::value_is_empty(get_primary_key_value_selected()) )
   {
     //Tell user that a primary key is needed to delete a record:
-    Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("No primary key value.")), true);
+    Gtk::MessageDialog dialog(Utils::bold_message(_("No primary key value.")), true);
     dialog.set_secondary_text(_("This record cannot be deleted because there is no primary key."));
     dialog.set_transient_for(*get_app_window());
     dialog.run();
@@ -446,7 +484,7 @@ void Box_Data_Details::on_button_del()
     if(confirm_delete_record())
     {
       const bool bTest = record_delete(m_primary_key_value);
-  
+
       if(bTest)
       {
         //Tell the list that it has been deleted:
@@ -480,6 +518,7 @@ void Box_Data_Details::on_button_nav_last()
   if(confirm_discard_unstored_data())
     signal_nav_last().emit();
 }
+#endif //GLOM_ENABLE_MAEMO
 
 Gnome::Gda::Value Box_Data_Details::get_entered_field_data(const sharedptr<const LayoutItem_Field>& field) const
 {
@@ -507,7 +546,7 @@ void Box_Data_Details::recalculate_fields_for_related_records(const Glib::ustrin
 
   //Check all fields in the parent table:
   const Gnome::Gda::Value primary_key_value = get_primary_key_value_selected();
-  for(type_vecFields::iterator iter = m_TableFields.begin(); iter != m_TableFields.end(); ++iter)
+  for(type_vec_fields::iterator iter = m_TableFields.begin(); iter != m_TableFields.end(); ++iter)
   {
     const sharedptr<const Field> field = *iter;
 
@@ -560,12 +599,13 @@ void Box_Data_Details::on_related_record_added(Gnome::Gda::Value /* strKeyValue
   }
   */
 
-  
+
 
   //Restore value:
   m_bDoNotRefreshRelated = bDoNotRefreshRelated;
 }
 
+#ifndef GLOM_ENABLE_MAEMO
 Box_Data_Details::type_signal_void Box_Data_Details::signal_nav_first()
 {
   return m_signal_nav_first;
@@ -588,8 +628,9 @@ Box_Data_Details::type_signal_void Box_Data_Details::signal_nav_last()
 
 Box_Data_Details::type_signal_record_deleted Box_Data_Details::signal_record_deleted()
 {
-  return m_signal_record_deleted; 
+  return m_signal_record_deleted;
 }
+#endif //GLOM_ENABLE_MAEMO
 
 Box_Data_Details::type_signal_requested_related_details Box_Data_Details::signal_requested_related_details()
 {
@@ -601,11 +642,11 @@ void Box_Data_Details::on_flowtable_layout_changed()
 {
   //Get new layout:
 #if 0
-  Document_Glom::type_list_layout_groups layout_groups;
+  Document::type_list_layout_groups layout_groups;
   m_FlowTable.get_layout_groups(layout_groups);
 
   //Store it in the document:
-  Document_Glom* document = get_document();
+  Document* document = get_document();
   if(document)
     document->set_data_layout_groups(m_layout_name, m_table_name, m_layout_platform, layout_groups);
   //Build the view again from the new layout:
@@ -613,11 +654,11 @@ void Box_Data_Details::on_flowtable_layout_changed()
   create_layout();
 
   //Store it in the document:
-  Document_Glom* document = get_document();
+  Document* document = get_document();
   if(document)
     document->set_modified();
-    
-    
+
+
   //And fill it with data:
   fill_from_database();
 }
@@ -641,7 +682,7 @@ void Box_Data_Details::on_flowtable_field_open_details_requested(const sharedptr
   if(Conversions::value_is_empty(field_value))
     return; //Ignore empty ID fields.
 
-  //If it's a simple field that is part of a relationship, 
+  //If it's a simple field that is part of a relationship,
   //identifying a related record.
   sharedptr<const Relationship> relationship = get_document()->get_field_used_in_relationship_to_one(m_table_name, layout_field);
   if(relationship)
@@ -653,12 +694,12 @@ void Box_Data_Details::on_flowtable_field_open_details_requested(const sharedptr
   //If it is a related field that is a primary key,
   //meaning it identifies a record in another table:
   sharedptr<const Field> field_info = layout_field->get_full_field_details();
-  const bool field_is_related_primary_key = 
-    layout_field->get_has_relationship_name() && 
+  const bool field_is_related_primary_key =
+    layout_field->get_has_relationship_name() &&
     field_info && field_info->get_primary_key();
   if(field_is_related_primary_key)
   {
-    signal_requested_related_details().emit(layout_field->get_table_used(m_table_name), field_value);   
+    signal_requested_related_details().emit(layout_field->get_table_used(m_table_name), field_value);
   }
 }
 
@@ -667,9 +708,14 @@ void Box_Data_Details::on_flowtable_script_button_clicked(const sharedptr<const
   if(layout_item)
   {
     const Gnome::Gda::Value primary_key_value = get_primary_key_value_selected();
+    const Glib::ustring table_name_before = m_table_name;
     execute_button_script(layout_item, primary_key_value);
 
-    //Refresh the view, in case the script changed any data:
+    //Refresh the view, in case the script changed any data,
+    //but not if the script navigated away:
+    if(m_table_name != table_name_before)
+      return;
+
     //(m_primary_key_value seems to be NULL here. We can use primary_key_value instead, but it's a bit strange. murrayc.)
     if(get_primary_key_is_in_foundset(m_found_set, primary_key_value)) //Check, because maybe the script deleted the current record, or changed something so that it should no longer be shown in the found set.
     {
@@ -677,12 +723,14 @@ void Box_Data_Details::on_flowtable_script_button_clicked(const sharedptr<const
     }
     else
     {
+      #ifndef GLOM_ENABLE_MAEMO
       //Tell the parent to do something appropriate, such as show another record:
       signal_record_deleted().emit(primary_key_value);
+      #endif
     }
   }
 }
-     
+
 void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutItem_Field>& layout_field, const Gnome::Gda::Value& field_value)
 {
   if(m_ignore_signals)
@@ -692,7 +740,7 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
 
   Gtk::Window* window = get_app_window();
 
-  Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
+  Document* document = dynamic_cast<Document*>(get_document());
 
   Gnome::Gda::Value primary_key_value = get_primary_key_value_selected();
   //std::cout << "Box_Data_Details::on_flowtable_field_edited(): primary_key_value=" << primary_key_value.to_string() << std::endl;
@@ -756,13 +804,13 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
       //Revert to the value in the database:
       const Gnome::Gda::Value value_old = get_field_value_in_database(field_in_record, window);
       set_entered_field_data(layout_field, value_old);
-   
-      return; 
+
+      return;
     }
 
     //Set the value in all instances of this field in the layout (The field might be on the layout more than once):
-    //if(layout_field->get_glom_type() != Field::TYPE_IMAGE) //TODO For now, don't do this for images, because the ImageGlom widget expects a broken GdaValue, because gda_value_get_binary() needs to be fixed.
-    m_FlowTable.set_field_value(layout_field, field_value);
+    //We don't need to set the value in the layout_field itself, as this is where the value comes from.
+    m_FlowTable.set_other_field_value(layout_field, field_value);
 
     //Update the field in the record (the record with this primary key):
 
@@ -802,8 +850,8 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
         //If this is a foreign key then refresh the related records:
         /*
         bool bIsForeignKey = false;
-        Document_Glom::type_vecRelationships vecRelationships = get_document()->get_relationships(m_table_name);
-        for(Document_Glom::type_vecRelationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); iter++)
+        Document::type_vec_relationships vecRelationships = get_document()->get_relationships(m_table_name);
+        for(Document::type_vec_relationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); ++iter)
         {
           const Relationship& relationship = *iter;
 
@@ -840,7 +888,7 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
       if(strFieldName == m_field_primary_key->get_name()) //If edited field is the primary key.
       {
         //Warn user that they can't choose their own primary key:
-        Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Primary key auto increments")), true);
+        Gtk::MessageDialog dialog(Utils::bold_message(_("Primary key auto increments")), true);
         dialog.set_secondary_text(_("The primary key is auto-incremented.\n You may not enter your own primary key value."));
         dialog.set_transient_for(*get_app_window());
         dialog.run();
@@ -889,7 +937,7 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
 void Box_Data_Details::on_userlevel_changed(AppState::userlevels user_level)
 {
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-  m_design_mode = ( user_level == AppState::USERLEVEL_DEVELOPER );    
+  m_design_mode = ( user_level == AppState::USERLEVEL_DEVELOPER );
   m_FlowTable.set_design_mode(m_design_mode);
   // Recreate the layout to correctly set the size of empty flowtables:
   create_layout();
@@ -986,8 +1034,8 @@ void Box_Data_Details::print_layout()
     //The groups:
     xmlpp::Element* nodeParent = nodeRoot;
 
-    Document_Glom::type_list_layout_groups layout_groups = get_data_layout_groups(m_layout_name, m_layout_platform);
-    for(Document_Glom::type_list_layout_groups::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
+    Document::type_list_layout_groups layout_groups = get_data_layout_groups(m_layout_name, m_layout_platform);
+    for(Document::type_list_layout_groups::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
     {
       sharedptr<const LayoutGroup> layout_group = *iter;
       print_layout_group(nodeParent, layout_group);
@@ -1000,7 +1048,7 @@ void Box_Data_Details::print_layout()
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 Dialog_Layout* Box_Data_Details::create_layout_dialog() const
 {
-  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "window_data_layout"); //TODO: Use a generic layout dialog?
+  Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "window_data_layout"); //TODO: Use a generic layout dialog?
   if(refXml)
   {
     Dialog_Layout_Details* dialog = 0;
@@ -1008,7 +1056,7 @@ Dialog_Layout* Box_Data_Details::create_layout_dialog() const
     return dialog;
   }
 
-  return NULL;
+  return 0;
 }
 
 void Box_Data_Details::prepare_layout_dialog(Dialog_Layout* dialog)
@@ -1027,4 +1075,9 @@ void Box_Data_Details::show_layout_toolbar(bool show)
 
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
+void Box_Data_Details::do_new_record()
+{
+  on_button_new();
+}
+
 } //namespace Glom
